%% %%%%%%%%%%%%%%%%%%%% INITAL VALUES %%%%%%%%%%%%%%%%%%%%%%%%

% Simulation model
y_0_sim = 0;
v_0_sim = 0;
p_B_0_sim = 1e5;          
p_A_0_sim = 2e5;
T_A_0_sim = 293;
T_B_0_sim = 293;
T_in_A_sim = 330;
T_in_B_sim = 293;
T_w_sim = 293;
y_ub_sim = 45e-3;
y_lb_sim = 0e-3;
Y_lb = 0.01;
G_lb = 1e-4;
w_1_0 = 0;

% Reference filter
y_d10 = 0;
y_d20 = 0;
y_d30 = 0;
y_d40 = 0;

%% %%%%%%%%%%%%%%%%%%%% REFERENCE INPUT %%%%%%%%%%%%%%%%%%%%%%%%
% Case Mode, X_r:
% 1 = External Input
% 2 = Square Wave
% 3 = Sine Wave
% 4 = Clutch Sequence
% 5 = Test Sequence
% 6 = Constant Input
% 7 = Ramp Wave
% 8 = Resettable ClutchSequence
% 9 = Manual Engange/Disengage
X_r=4;
X_disengage = 1;    % Always start at 1.   
T_r=2.5;            % Period [s]
S_r=1e-3;           % Reference input scaling
R_0=1;            % Bias
R_1=14;           % Amplitude
T_delay=0;          % Time delay [s]
L_slip=9.0;         % Location, slip point center
W_slip=4.0;         % Width, slip point
reset_cs=1;         % Reset resattable clutchsequence.
%                   % This clutchSequence has 2 sec delay.


%% %%%%%%%%%%%%%%%%%%%% PHYSICAL PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%
%%% Parameters for controller and initial settings %%%
%%% Parameters for clutch systems are set in "mask"

% Reference values
T_0 = 293;           % [K]
P_0 = 1e5;           % [bar]
P_E = P_0;        % Exhaust pressure [bar]
P_s=9.5e5;           % Pressure, supply [Pa]
R = 288;                % Gas constant of air [J/(kg*K)]
rho_0 = 1.185;       % Air density, ISO reference condition [kg/m^3]
% Clutch Actuator
A = 12.3e-3;        % [m]
V_0 = 0.8e-3;      % Volume at y=0, chamber A [m^3]
M = 10;              % [kg]
% Friction parameters
D_v	= 3.071912421740924e+003;
K_f 	= 4.659530303747875e+004;
D_f 	= 1.000000131326377e+002;
F_d 	= 2.099999993757047e+002;
y_f_0=1/K_f*F_d*sign(v_0_sim);    % Average deflection [?]
%Flow paraeters
C_v=	2.895032495811626e-008;
tau_v= 	0.03833788012589;
B_0 = 0.528;
Tau_w = 0.05;

% Load characteristics
theta_l = 1.0e+003 *[  -0.592094643997316   .05111147973241 2.00000000000085   2.63759736205582   3.20832821188495 3.16062161416013   2.95958874731289   2.95585608064665  2.96824124238484   2.98290116473576]';
d_l =[   0.00699951204684   0.00475291228824   0.00699972558903 0.00700017245381   0.00749955164514   0.00739759618261 0.00640862073634   0.00812905992460   0.00489113725834   0.00751633405395]'; 
w_l=3./d_l;                                 
y_l=linspace(-5,25,200)*1e-3;  
c_l=linspace(-2e-3,25e-3,10);
w_l=3./d_l;                                 
y_l=linspace(-5,25,200)*1e-3;  
[Phi_l,dPhi_l,ddPhi_l]=phi_M_gaussian3(y_l,c_l,w_l);%[Phi_l,dPhi_l,ddPhi_l]=phi_gauss_n(y_l,c_l,w_l);            % Basis functions
Mphi_l = [y_l; Phi_l];
dMphi_l = [y_l; dPhi_l];
ddMphi_l = [y_l; ddPhi_l];
%Calculate zeta_0 from p_0
%zeta_0 = (V_0 + y_0_design*A)*p_0_controller;
epsilon = 1e-4;  %For smooth approximation of abs()

% Curvefit exponential load model
bias = 0;
sim('LoadChar')
t = F_l.time;
y = y_out.signals.values;
F_load = F_l.signals.values;
F_load_d = F_l_d.signals.values;

x_0 = [2500 300 10000];
[x1,resnorm] = lsqcurvefit(@loadFunc,x_0,y,F_load);


bias = -1000;;
sim('LoadChar')
t = F_l.time;
y = y_out.signals.values;
F_load_lb = F_l.signals.values;
F_load_d_lb = F_l_d.signals.values;

x_0 = [2500 300 10000];
[x2,resnorm] = lsqcurvefit(@loadFunc,x_0,y,F_load_lb) ;


bias = 1000;
sim('LoadChar')
t = F_l.time;
y = y_out.signals.values;
F_load_ub = F_l.signals.values;
F_load_d_ub = F_l_d.signals.values;

x_0 = [2500 300 10000];
[x3,resnorm] = lsqcurvefit(@loadFunc,x_0,y,F_load_ub) ;

%% VALVE MODEL %%
% Flow rate characteristic (simulation model):
B_vss=0.6;          % Critical pressure ratio parameter, valve [-]
B_ves=-0.8;

C_vss=7.7e-9;   
C_ves=9.2e-9;

% PWM characteristics, control ranges, duty cycle ratio:
% u_v axis:
u_vsDat=[0.00	0.25    0.85    0.90]';
u_veDat=[0.00	0.25    0.85    0.90]';

% y_v axis:
y_vsDat=[0.00  0.18    0.90    1.00]';
y_veDat=[0.00  0.18    0.90    1.00]';


g_v_lb=1e-4;
u_v_ub=0.95;
y_v_lb=0.01;

R0_vss = 0.0;
R0_ves = 0.0;

R1_vss = 0.65;
R1_ves = 0.7;

Tau_w = 0.05;

%% SIMULATION SETTINGS %%%

D_tw=10;           % Downsampling, simulation
N_tw=inf;          % Maximum number of samples
D_par_est=10;       % Downsampling of parameter estimates

y = 0;
v = 0;
a = 0;

%% %%%%%%%%%%%%%%%%%%%% Controller Initialization %%%%%%%%%%%%%%%%%%%%%%%%

% Sample time 
dT = 0.002;
dT_sim = 0.002;

% Simulation length
T_sim = 10;

% Simulation parameters
noiseSelect = 1;            % 0 = no noise, 1 = noise added to y_m
noisePower = 10^(-11.5);    % Power of the measurment noise

% Controller constants
epsilon_boundary = 8;  % Boundary layer width
k = 0.017;   % Controller gain
a1 = 1050;    
a2 = 95;    % Constants describing the Sliding Surface S(t)

L_v = 175;
L_a = 80;
y_0_filt = 0;
v_0_filt = 0;
a_0_filt = 0;


% Parameters for Relax Valves
Topen_ll = 1;           % [s] exhaust valves open when below threshold
threshold_ll = 3e-3;    % [m] 

% Viscous friction parameter
D = 3000;   

% Parameters for flow model inverse:
B_vss_con=0.6;          % Critical pressure ratio parameter, valve [-]
B_ves_con=-0.8;

C_vss_con=7.7e-9;   
C_ves_con=9.2e-9;

C_vss=7.7e-9;   
C_ves=9.2e-9;

R0_vss_con = 0.0;
R0_ves_con = 0.0;

R1_vss_con = 0.65;
R1_ves_con = 0.7;

R0_vss = 0.0;
R0_ves = 0.0;

R1_vss = 0.65;
R1_ves = 0.7;

% Flow restrictions
flow_o_max = rho_0*C_ves*P_s;
flow_i_max = rho_0*C_vss*P_s;

% Pole placement reference trajectory filter
L_r=50;             

dT_est = 0.002;
